"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCypressTestBed = exports.mountHtml = exports.initEnvHtml = exports.mount = exports.initEnv = exports.setConfig = void 0;
var testing_1 = require("@angular/core/testing");
var testing_2 = require("@angular/platform-browser-dynamic/testing");
var config_1 = require("./config");
var css_utils_1 = require("./css-utils");
var proxy_component_1 = require("./proxy.component");
var config = new config_1.CypressAngularConfig();
function setConfig(c) {
    config = c;
}
exports.setConfig = setConfig;
function init(component, options) {
    Cypress.log({ displayName: 'Unit Test', message: ['Init Environment'] });
    checkIsComponentSpec();
    testing_1.TestBed.resetTestEnvironment();
    testing_1.TestBed.initTestEnvironment(testing_2.BrowserDynamicTestingModule, testing_2.platformBrowserDynamicTesting());
    var declarations = [component];
    var providers = [];
    // automatic component change detection
    if (config.detectChanges) {
        providers.push({ provide: testing_1.ComponentFixtureAutoDetect, useValue: true });
    }
    if (options) {
        config = __assign(__assign({}, config), options);
        if (options.declarations) {
            declarations.push.apply(declarations, options.declarations);
        }
        if (options.providers) {
            providers.push.apply(providers, options.providers);
        }
    }
    testing_1.TestBed.configureTestingModule({
        declarations: declarations,
        imports: options ? options.imports : [],
        providers: providers,
        schemas: options ? options.schemas : [],
    });
    // @ts-ignore
    var document = cy.state('document');
    var el = document.getElementById('root');
    if (el === null) {
        throw new Error('root element not found');
    }
    css_utils_1.injectStylesBeforeElement(config, document, el);
}
function initEnv(component, options) {
    init(component, options);
    testing_1.TestBed.compileComponents();
}
exports.initEnv = initEnv;
function mount(component, inputs) {
    checkIsComponentSpec();
    // TODO improve logging using a full log instance
    Cypress.log({
        displayName: 'Unit Test',
        message: ["Mounting **" + component.name + "**"],
    });
    var fixture = testing_1.TestBed.createComponent(component);
    var componentInstance = fixture.componentInstance;
    componentInstance = Object.assign(componentInstance, inputs);
    if (config.detectChanges) {
        fixture.whenStable().then(function () { return fixture.detectChanges(); });
        fixture.detectChanges();
    }
    return fixture;
}
exports.mount = mount;
function initEnvHtml(component, options) {
    if (options) {
        if (options.declarations) {
            options.declarations.push(component);
        }
        else {
            options.declarations = [component];
        }
    }
    else {
        options = { declarations: [component] };
    }
    init(proxy_component_1.ProxyComponent, options);
}
exports.initEnvHtml = initEnvHtml;
function mountHtml(htmlTemplate) {
    checkIsComponentSpec();
    Cypress.log({
        displayName: 'Unit Test',
        message: ["Mounting **" + htmlTemplate + "**"],
    });
    testing_1.TestBed.overrideComponent(proxy_component_1.ProxyComponent, {
        set: { template: htmlTemplate },
    });
    testing_1.TestBed.compileComponents();
    var fixture = testing_1.TestBed.createComponent(proxy_component_1.ProxyComponent);
    if (config.detectChanges) {
        fixture.whenStable().then(function () { return fixture.detectChanges(); });
        fixture.detectChanges();
    }
    return fixture;
}
exports.mountHtml = mountHtml;
function getCypressTestBed() {
    return testing_1.getTestBed();
}
exports.getCypressTestBed = getCypressTestBed;
function checkIsComponentSpec() {
    if (!isComponentSpec()) {
        throw new Error('Angular component test from an integration spec is not allowed');
    }
}
// @ts-ignore
function isComponentSpec() {
    return Cypress.spec.specType === 'component';
}
